<?php

namespace Imoje\Blik\Block;

include_once __DIR__ . "/../../Libs/autoload.php";

use Imoje\Blik\Model\Order;
use Imoje\Payment\Api;
use Imoje\Payment\Helper;
use Imoje\Payment\Invoice;
use Imoje\Payment\Util;
use Magento\Checkout\Helper\Cart;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\App\ResponseFactory;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Asset\Repository as AssetRepository;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\ScopeInterface;
use Zend\Log\Logger;
use Zend\Log\Writer\Stream;
use Magento\Sales\Model\ResourceModel\Order\Tax\Item as TaxItem;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Tax\Api\TaxClassRepositoryInterface;

/**
 * Class Checkout
 *
 * @package Imoje\Blik\Block
 */
class Checkout extends Template
{

	/**
	 * @var UrlInterface
	 */
	public $urlBuilder;

	/**
	 * @var ScopeConfigInterface
	 */
	protected $scopeConfig;

	/**
	 * @var Session\
	 */
	protected $customerSession;

	/**
	 * @var Registry
	 */
	private $registry;

	/**
	 * @var Order
	 */
	private $order;

	/**
	 * @var ResponseFactory
	 */
	private $responseFactory;

	/**
	 * @var ProductMetadataInterface $productMetadataInterface
	 */
	private $productMetadataInterface;

	/**
	 * @var AssetRepository
	 */
	private $assetRepository;

	/**
	 * @var TaxItem $taxItem
	 */
	private $taxItem;

	/**
	 * @var ProductRepositoryInterface\ $productRepository
	 */
	protected $productRepository;

	/**
	 * @var TaxClassRepositoryInterface $taxClassRepository
	 */
	protected $taxClassRepository;

	/**
	 * @var StoreInterface $store
	 */
	protected $store;

	/**
	 * Checkout constructor.
	 *
	 * @param Session                     $session
	 * @param Context                     $context
	 * @param Order                       $order
	 * @param Registry                    $registry
	 * @param StoreInterface              $store
	 * @param UrlInterface                $urlBuilder
	 * @param ScopeConfigInterface        $scopeConfig
	 * @param ResponseFactory             $responseFactory
	 * @param ProductMetadataInterface    $productMetadataInterface
	 * @param AssetRepository             $assetRepository
	 * @param TaxItem                     $taxItem
	 * @param ProductRepositoryInterface  $productRepository
	 * @param TaxClassRepositoryInterface $taxClassRepository
	 */
	public function __construct(
		Session                     $session,
		Context                     $context,
		Order                       $order,
		Registry                    $registry,
		StoreInterface              $store,
		UrlInterface                $urlBuilder,
		ScopeConfigInterface        $scopeConfig,
		ResponseFactory             $responseFactory,
		ProductMetadataInterface    $productMetadataInterface,
		AssetRepository             $assetRepository,
		TaxItem                     $taxItem,
		ProductRepositoryInterface  $productRepository,
		TaxClassRepositoryInterface $taxClassRepository
	) {
		parent::__construct($context);
		$this->registry = $registry;
		$this->order = $order;
		$this->store = $store;
		$this->urlBuilder = $urlBuilder;
		$this->scopeConfig = $scopeConfig;
		$this->customerSession = $session;
		$this->responseFactory = $responseFactory;
		$this->productMetadataInterface = $productMetadataInterface;
		$this->assetRepository = $assetRepository;
		$this->taxItem = $taxItem;
		$this->productRepository = $productRepository;
		$this->taxClassRepository = $taxClassRepository;

		$paramBlikCode = $this->getRequest()->getParam('blikCode');

		if($paramBlikCode) {
			$this->renderForm($paramBlikCode);
			die();
		}

		if(!$this->getOrderState($this->getOrderIdFromPost())
			|| $this->getOrderState($this->getOrderIdFromPost()) === \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT) {
			$this->redirectToHomePage();
			die();
		}
	}

	/**
	 * @param string $paramBlikCode
	 *
	 * @return string
	 */
	public function renderForm($paramBlikCode = '')
	{

		$writer = new Stream(BP . '/var/log/imoje.log');
		$this->logger = new Logger();
		$this->logger->addWriter($writer);

		if($paramBlikCode
			&& (!is_numeric($_POST['blikCode'])
				|| strlen($_POST['blikCode']) != 6)) {

			$this->logger->info('Blik code is invalid');

			echo json_encode([
				'status' => false,
			]);
			die();
		}

		$imojeApi = new Api(
			$this->getConfigValue('payment/imoje_blik/authorization_token'),
			$this->getConfigValue('payment/imoje_blik/merchant_id'),
			$this->getConfigValue('payment/imoje_blik/service_id'),
			$this->getConfigValue('payment/imoje_blik/sandbox')
				? Util::ENVIRONMENT_SANDBOX
				: Util::ENVIRONMENT_PRODUCTION
		);

		$order = $this->getOrderObject($this->getOrderId());

		$email = $order->getCustomerEmail();

		$billingAddress = $order->getBillingAddress();

		$prepareData = $imojeApi->prepareData(
			$order->getGrandTotal(),
			$order->getOrderCurrency()->getCode(),
			$order->getIncrementId(),
			Util::getPaymentMethod('blik'),
			Util::getPaymentMethodCode('blik'),
			$this->urlBuilder->getUrl('checkout/onepage/success/'),
			$this->urlBuilder->getUrl('checkout/onepage/failure/'),
			$billingAddress->getFirstname(),
			$billingAddress->getLastname(),
			$email,
			Api::TRANSACTION_TYPE_SALE,
			'',
			$paramBlikCode,
			[],
			'',
			$this->getConfigValue('payment/imoje_blik/ing_ksiegowosc')
				? Invoice::get(
				Helper::getCart(
					$order,
					$this->taxItem,
					$this->productRepository,
					$this->taxClassRepository
				),
				$email,
				true
			)
				: ''
		);

		$transaction = $imojeApi->createTransaction($prepareData);

		$order->setState(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT);
		$order->save();

		if($transaction['success']) {

			if($paramBlikCode) {
				echo json_encode([
					'status' => $transaction['success'],
				]);
				die();
			}

			return Util::createOrderForm(
				Api::parseStringToArray($transaction['body']),
				$transaction['body']['action']['url'],
				$transaction['body']['action']['method'],
				__('Continue')
			);
		}

		if(isset($transaction['data']['body']) && $transaction['data']['body']) {
			$this->logger->info('Transaction could not be initialized, error: ' . $transaction['data']['body']);
		}

		if($paramBlikCode) {
			echo json_encode([
				'status' => false,
			]);
			die();
		}

		$this->redirectToHomePage();
		die();
	}

	/**
	 * @param string $value
	 *
	 * @return string
	 */
	private function getConfigValue($value)
	{
		return $this->scopeConfig->getValue($value, ScopeInterface::SCOPE_STORE);
	}

	/**
	 * Render form with order information prepared for Payment Page
	 *
	 * @param string $id
	 *
	 * @return bool|\Imoje\Blik\Model\Sales\Order
	 */
	public function getOrderObject($id = '')
	{
		if(empty($id)) {
			$id = $this->getOrderId();
		}

		return $this->order->loadOrderById($id);
	}

	/**
	 * @return integer
	 */
	public function getOrderId()
	{
		return $this->registry->registry('orderId');
	}

	/**
	 * That function redirect to homepage
	 *
	 * @return mixed
	 */
	public function redirectToHomePage()
	{
		return $this->responseFactory->create()->setRedirect($this->urlBuilder->getUrl())->sendResponse(); //Redirect to cms page
	}

	/**
	 * @param string $orderId
	 *
	 * @return mixed
	 */
	public function getOrderState($orderId = '')
	{

		if($this->getOrderObject($orderId)) {
			return $this->getOrderObject($orderId)->getState();
		}

		return null;
	}

	/**
	 * @return string
	 */
	public function getOrderIdFromPost()
	{
		$id = '';
		if(isset($_POST['orderId'])) {
			$id = $_POST['orderId'];
		}

		return $id;
	}

	/**
	 * @return string
	 */
	public function isCodeField()
	{
		return $this->getConfigValue('payment/imoje_blik/blik_code_field');
	}

	/**
	 * @return int[]
	 */
	public function getOneClickVariables()
	{

		$loadingGif = $this->assetRepository->getUrl('Imoje_Blik::images/loading.gif');
		$paymentLink = $this->getBaseUrl() . 'imoje_blik/payment/oneclick';
		//
		//if(
		//	!$this->getConfigValue('payment/imoje_blik/oneclick')
		//	|| !$this->customerSession->isLoggedIn()
		//) {
		//	return [
		//		'profileId'     => '',
		//		'activeProfile' => '',
		//		'paymentLink'   => $paymentLink,
		//		'orderId'       => '',
		//		'returnUrl'     => '',
		//		'availability'  => false,
		//		'loadingGif'    => $loadingGif,
		//	];
		//}

		return [
			'profileId'     => '',
			'activeProfile' => '',
			'paymentLink'   => $paymentLink,
			'orderId'       => $this->getOrderObject($this->getOrderId())->getIncrementId(),
			'returnUrl'     => $this->urlBuilder->getUrl('checkout/onepage/success/'),
			'availability'  => false,
			'loadingGif'    => $loadingGif,
		];

		$activeProfile = [];
		$profileId = '';

		$imojeApi = new Api(
			$this->getConfigValue('payment/imoje_blik/authorization_token'),
			$this->getConfigValue('payment/imoje_blik/merchant_id'),
			$this->getConfigValue('payment/imoje_blik/service_id'),
			$this->getConfigValue('payment/imoje_blik/sandbox')
				? Util::ENVIRONMENT_SANDBOX
				: Util::ENVIRONMENT_PRODUCTION
		);

		$profileList = $imojeApi->getBlikProfileList(
			Util::getCid($this->customerSession->getCustomer()->getId() . $this->customerSession->getCustomer()->getEmail())
		);

		if((isset($profileList['success']) && $profileList['success'])
			&& $profileList['body']['blikProfiles']) {

			foreach($profileList['body']['blikProfiles'] as $profile) {

				if($profile['isActive']) {

					$isProfileLabel = isset($profile['aliasList'][1]);

					foreach($profile['aliasList'] as $k => $v) {

						if(!$isProfileLabel) {

							$profile['aliasList'][$k]['label'] = $profile['label'];
						}
					}

					$profileId = $profile['id'];
					$activeProfile = $profile;
					break;
				}
			}
		}

		return [
			'profileId'     => $profileId,
			'activeProfile' => $activeProfile,
			'paymentLink'   => $paymentLink,
			'orderId'       => $this->getOrderObject($this->getOrderId())->getIncrementId(),
			'returnUrl'     => $this->urlBuilder->getUrl('checkout/onepage/success/'),
			'availability'  => true,
			'loadingGif'    => $loadingGif,
		];
	}
}
