<?php

namespace Imoje\Blik\Controller\Payment;

include_once __DIR__ . "/../../../Libs/autoload.php";

use Exception;
use Imoje\Payment\Api;
use Imoje\Payment\Helper;
use Imoje\Payment\Util;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Magento\Framework\UrlInterface;
use Magento\Sales\Model\Order\Email\Sender\InvoiceSender;
use Magento\Sales\Model\Order\Invoice;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;
use Magento\Sales\Model\OrderRepository;
use Magento\Sales\Model\Service\InvoiceService;
use Magento\Store\Model\ScopeInterface;
use Magento\Sales\Model\ResourceModel\Order\Tax\Item as TaxItem;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Tax\Api\TaxClassRepositoryInterface;

/**
 * Class Oneclick
 *
 * @package Imoje\Blik\Controller\Payment
 */
class Oneclick extends Action
{

	/**
	 * @var string
	 */
	const ACTION_CREATE_TRANSACTION = 'createTransaction';

	/**
	 * @var string
	 */
	const ACTION_CHECK_TRANSACTION = 'checkTransaction';

	/**
	 * @var string
	 */
	const ACTION_DEBIT_ALIAS = 'debitAlias';

	/**
	 * @var string
	 */
	const ACTION_DEACTIVATE_ALIAS = 'deactivateAlias';

	/**
	 * @var UrlInterface
	 */
	public $urlBuilder;

	/**
	 * @var Session
	 */
	protected $customerSession;

	/**
	 * @var Api
	 */
	protected $imojeApi;

	/**
	 * @var Order
	 */
	protected $order;

	/**
	 * @var ScopeConfigInterface
	 */
	protected $scopeConfig;

	/**
	 * @var BuilderInterface
	 */
	protected $builderInterface;

	/**
	 * @var OrderRepository
	 */
	protected $orderRepository;

	/**
	 * @var InvoiceService
	 */
	protected $invoiceService;

	/**
	 * @var \Magento\Framework\DB\Transaction
	 */
	protected $dbTransaction;

	/**
	 * @var Magento\Sales\Model\Order\Email\Sender\InvoiceSender
	 */
	protected $invoiceSender;

	/**
	 * @var TaxItem $taxItem
	 */
	private $taxItem;

	/**
	 * @var ProductRepositoryInterface\ $productRepository
	 */
	protected $productRepository;

	/**
	 * @var TaxClassRepositoryInterface $taxClassRepository
	 */
	protected $taxClassRepository;

	/**
	 * Oneclick constructor.
	 *
	 * @param Session                           $session
	 * @param Context                           $context
	 * @param Order                             $order
	 * @param ScopeConfigInterface              $scopeConfig
	 * @param BuilderInterface                  $builderInterface
	 * @param OrderRepositoryInterface          $orderRepository
	 * @param InvoiceService                    $invoiceService
	 * @param \Magento\Framework\DB\Transaction $dbTransaction
	 * @param InvoiceSender                     $invoiceSender
	 * @param UrlInterface                      $urlBuilder
	 * @param TaxItem                           $taxItem
	 * @param ProductRepositoryInterface        $productRepository
	 * @param TaxClassRepositoryInterface       $taxClassRepository
	 */
	public function __construct(
		Session                           $session,
		Context                           $context,
		Order                             $order,
		ScopeConfigInterface              $scopeConfig,
		BuilderInterface                  $builderInterface,
		OrderRepositoryInterface          $orderRepository,
		InvoiceService                    $invoiceService,
		\Magento\Framework\DB\Transaction $dbTransaction,
		InvoiceSender                     $invoiceSender,
		UrlInterface                      $urlBuilder,
		TaxItem                           $taxItem,
		ProductRepositoryInterface        $productRepository,
		TaxClassRepositoryInterface       $taxClassRepository
	) {
		parent::__construct($context);
		$this->customerSession = $session;
		$this->order = $order;
		$this->scopeConfig = $scopeConfig;
		$this->builderInterface = $builderInterface;
		$this->orderRepository = $orderRepository;
		$this->invoiceService = $invoiceService;
		$this->dbTransaction = $dbTransaction;
		$this->invoiceSender = $invoiceSender;
		$this->urlBuilder = $urlBuilder;
		$this->taxItem = $taxItem;
		$this->productRepository = $productRepository;
		$this->taxClassRepository = $taxClassRepository;

		$this->imojeApi = new Api(
			$this->getConfigValue('payment/imoje_blik/authorization_token'),
			$this->getConfigValue('payment/imoje_blik/merchant_id'),
			$this->getConfigValue('payment/imoje_blik/service_id'),
			$this->getConfigValue('payment/imoje_blik/sandbox')
				? Util::ENVIRONMENT_SANDBOX
				: Util::ENVIRONMENT_PRODUCTION
		);
	}

	/**
	 * @param string $value
	 *
	 * @return string
	 */
	private function getConfigValue($value)
	{
		return $this->scopeConfig->getValue($value, ScopeInterface::SCOPE_STORE);
	}

	/**
	 * @return void
	 * @throws Exception
	 */
	public function execute()
	{

		if(empty($_REQUEST['action'])) {
			self::doResponseJson();
		}

		switch($_REQUEST['action']) {
			case self::ACTION_CREATE_TRANSACTION:
				$this->createTransaction();

				return;
			case self::ACTION_CHECK_TRANSACTION:
				$this->checkTransaction();

				return;
			case self::ACTION_DEBIT_ALIAS:
				$this->debitAlias();

				return;
			case self::ACTION_DEACTIVATE_ALIAS:
				$this->deactivateAlias();

				return;
			default:
				self::doResponseJson();
		}
	}

	/**
	 * @param array $array
	 * @param bool  $success
	 */
	public static function doResponseJson($array = [], $success = false)
	{

		$array['success'] = $success;

		header('Content-Type: application/json');

		echo json_encode($array);
		die();
	}

	/**
	 * Function that create transaction in api imoje
	 */
	private function createTransaction()
	{

		if(empty($_REQUEST['orderId'])) {
			self::doResponseJson();
		}

		$blikCode = null;

		$isCode = isset($_REQUEST['code']) && $_REQUEST['code'];

		if($isCode
			&& isset($_REQUEST['orderId']) && $_REQUEST['orderId']) {

			if(!is_numeric($_REQUEST['code'])
				|| strlen($_REQUEST['code']) != 6) {

				self::doResponseJson();
			}

			$blikCode = $_REQUEST['code'];
		}

		$order = $this->order->loadByIncrementId($_REQUEST['orderId']);

		if(!$order) {
			self::doResponseJson();
		}

		$paymentMethod = '';
		$paymentMethodCode = '';
		$transactionBody = '';
		$transactionUrl = '';

		if($blikCode) {
			$paymentMethod = Util::getPaymentMethod('blik');
			$paymentMethodCode = Util::getPaymentMethodCode('blik');

			$isRememberBlikCode = isset($_REQUEST['rememberBlikCode']) && $_REQUEST['rememberBlikCode'];

			if($isRememberBlikCode) {
				$paymentMethodCode = Util::getPaymentMethodCode('blik_oneclick');
			}

			if($isRememberBlikCode && $isCode
				&& isset($_REQUEST['profileId']) && $_REQUEST['profileId']) {

				$transactionBody = $this->imojeApi->prepareBlikOneclickData(
					$_REQUEST['profileId'],
					$order->getGrandTotal(),
					$order->getOrderCurrencyCode(),
					$_REQUEST['orderId'],
					$_REQUEST['orderId'],
					$_SERVER['REMOTE_ADDR'],
					'',
					$blikCode
				);
				$transactionUrl = $this->imojeApi->getDebitBlikProfileUrl();
			}
		}

		$cid = '';
		if($this->customerSession->isLoggedIn()) {

			$cid = Util::getCid($this->customerSession->getCustomer()->getId() . $this->customerSession->getCustomer()->getEmail());
		}

		if(!$transactionBody) {

			$billingAddress = $order->getBillingAddress();

			$email = $order->getCustomerEmail();

			$transactionBody = $this->imojeApi->prepareData(
				$order->getGrandTotal(),
				$order->getOrderCurrencyCode(),
				$_REQUEST['orderId'],
				$paymentMethod,
				$paymentMethodCode,
				$this->urlBuilder->getUrl('checkout/onepage/success/'),
				$this->urlBuilder->getUrl('checkout/onepage/failure/'),
				$billingAddress->getFirstname(),
				$billingAddress->getLastname(),
				$email,
				Api::TRANSACTION_TYPE_SALE,
				'',
				$blikCode,
				'',
				$cid,
				$this->getConfigValue('payment/imoje_blik/ing_ksiegowosc')
					? \Imoje\Payment\Invoice::get(
					Helper::getCart(
						$order,
						$this->taxItem,
						$this->productRepository,
						$this->taxClassRepository
					),
					$email,
					true)
					: ''
			);
		}

		$transaction = $this->imojeApi->createTransaction($transactionBody, $transactionUrl);

		if($transaction['success']) {

			self::doResponseJson($this->prepareSuccessResponse($transaction), true);
		}

		self::doResponseJson();
	}

	/**
	 * @param array $transaction
	 *
	 * @return array
	 */
	private function prepareSuccessResponse($transaction)
	{

		$array = [
			'transaction' => [
				'status' => $transaction['body']['transaction']['status'],
				'id'     => $transaction['body']['transaction']['id'],
			],
		];

		if(isset($transaction['body']['transaction']['statusCode']) && $transaction['body']['transaction']['statusCode']) {
			$array['error'] = $this->getErrorMessage($transaction['body']['transaction']['statusCode']);
		}

		if(isset($transaction['body']['code']) && $transaction['body']['code']) {
			$array['code'] = $transaction['body']['code'];
		}

		if(isset($transaction['body']['newParamAlias']) && $transaction['body']['code']) {
			$array['newParamAlias'] = $transaction['body']['newParamAlias'];
		}

		return $array;
	}

	/**
	 * @param string $code
	 *
	 * @return string
	 */
	private function getErrorMessage($code)
	{

		$tEnterT6 = __('Insert BLIK code.');
		$tTryAgain = __('Please try again.');

		switch($code) {
			case 'BLK-ERROR-210000':
				return __('Payment failed.') . ' ' . $tTryAgain;
			case 'BLK-ERROR-210001':
				return __('Technical break in your bank. Pay later or use another bank\'s application.');
			case 'BLK-ERROR-210002':
				return __('Alias not found. To proceed the payment you need to pay with BLIK code.');
			case 'BLK-ERROR-210003':
			case 'BLK-ERROR-210004':
				return __('Alias declined. To proceed the payment you need to pay with BLIK code.');
			case 'BLK-ERROR-210005':
				$msg = __('You have entered wrong BLIK code.');
				break;
			case 'BLK-ERROR-210006':
				$msg = __('BLIK code expired.');
				break;
			case 'BLK-ERROR-210007':
			case 'BLK-ERROR-210008':
				$msg = __('Something went wrong with BLIK code.');
				break;
			case 'BLK-ERROR-210009':
				$msg = __('Payment declined at the banking application.');
				break;
			case 'BLK-ERROR-210010':
			case 'BLK-ERROR-210011':
				$msg = __('Payment failed - not confirmed on time in the banking application.');
				break;
			case 'BLK-ERROR-210012':
				$msg = __('Inserted wrong PIN code in banking application.');
				break;
			case 'BLK-ERROR-210013':
				$msg = __('Payment failed (security).');
				break;
			case 'BLK-ERROR-210014':
				$msg = __('Limit exceeded in your banking application.');
				break;
			case 'BLK-ERROR-210015':
				$msg = __('Insufficient funds in your bank account.');
				break;
			case 'BLK-ERROR-210016':
				$msg = __('Issuer declined.');
				break;
			case 'BLK-ERROR-210017':
				$msg = __('Transaction not found.');
				break;
			case 'BLK-ERROR-210018':
				$msg = __('Bad IBAN.');
				break;
			case 'BLK-ERROR-210019':
				$msg = __('Transfer not possible.');
				break;
			case 'BLK-ERROR-210020':
				$msg = __('Return late.');
				break;
			case 'BLK-ERROR-210021':
				$msg = __('Return amount exceeded.');
				break;
			case 'BLK-ERROR-210022':
				$msg = __('Transfer late.');
				break;
			default:
				return __('Payment failed.') . ' ' . $tEnterT6;
		}

		return $msg . ' ' . $tEnterT6;
	}

	/**
	 * Function that checks transaction in api imoje
	 */
	private function checkTransaction()
	{
		if(empty($_REQUEST['transactionUuid'])) {

			self::doResponseJson();
		}

		$transaction = $this->imojeApi->getTransaction($_REQUEST['transactionUuid']);

		if($transaction['success']) {
			self::doResponseJson($this->prepareSuccessResponse($transaction), true);
		}

		self::doResponseJson();
	}

	/**
	 * Function that debit alias in api imoje
	 */
	private function debitAlias()
	{

		if(empty($_REQUEST['orderId'])
			|| empty($_REQUEST['profileId'])
			|| empty($_REQUEST['blikKey'])
		) {
			self::doResponseJson();
		}

		$order = $this->order->loadByIncrementId($_REQUEST['orderId']);

		if(!$order) {
			self::doResponseJson();
		}

		$transaction = $this->imojeApi->createTransaction(
			$this->imojeApi->prepareBlikOneclickData(
				$_REQUEST['profileId'],
				$order->getGrandTotal(),
				$order->getOrderCurrencyCode(),
				$_REQUEST['orderId'],
				$_REQUEST['orderId'],
				$_SERVER['REMOTE_ADDR'],
				$_REQUEST['blikKey']
			),
			$this->imojeApi->getDebitBlikProfileUrl()
		);

		if($transaction['success']) {
			self::doResponseJson($this->prepareSuccessResponse($transaction), true);
		}

		self::doResponseJson();
	}

	/**
	 * Function that deactivate alias in api imoje
	 */
	private function deactivateAlias()
	{

		if(
			empty($_REQUEST['profileId'])
			|| empty($_REQUEST['blikKey'])
		) {
			self::doResponseJson();
		}

		$deactivate = $this->imojeApi->createTransaction(
			json_encode([
					'blikProfileId' => $_REQUEST['profileId'],
					'blikKey'       => $_REQUEST['blikKey'],
				]
			),
			$this->imojeApi->getDeactivateBlikProfileUrl()
		);

		if($deactivate['success']) {
			self::doResponseJson([], true);
		}

		self::doResponseJson();
	}
}
