<?php

namespace Imoje\Payment;

use Imoje\Paywall\Model\Order;
use Magento\Payment\Model\InfoInterface;
use Magento\Sales\Model\ResourceModel\Order\Tax\Item;
use Magento\Framework\Exception\LocalizedException;

class Helper
{

    /**
     * @param Order $order
     * @param Item $taxItem
     *
     * @return CartData
     */
    public static function getCart($order, $taxItem, $productRepository, $taxClassRepository)
    {

        $cartData = new CartData();

        foreach ($order->getAllItems() as $item) {


            $rate = (float)$item->getTaxPercent();

            if ($taxClassRepository
                    ->get($productRepository->getById($item->getProductId())
                        ->getData('tax_class_id'))
                    ->getClassName() === Invoice::SHOP_TAX_EXEMPT) {
                $rate = explode('_', Invoice::TAX_EXEMPT)[1];
            }

            $cartData->addItem(
                $item->getId(),
                $rate,
                $item->getName(),
                Util::convertAmountToFractional($item->getPriceInclTax()),
                (float)$item->getQtyOrdered()
            );
        }

        $addressBilling = $order->getBillingAddress();
        $addressDelivery = $order->getShippingAddress();

        $shippingTaxRate = '';

        foreach ($taxItem->getTaxItemsByOrderId($order->getId()) as $item) {
            if ($item['taxable_item_type'] === 'shipping') {
                $shippingTaxRate = $item['tax_percent'];
            }
        }

        $cartData->setShipping(
            (float)$shippingTaxRate,
            $order->getShippingDescription(),
            Util::convertAmountToFractional($order->getShippingInclTax())
        );

        $cartData->setAddressDelivery(
            $addressDelivery->getCity(),
            $addressDelivery->getFirstname() . ' ' . $addressDelivery->getLastname(),
            $addressDelivery->getTelephone(),
            $addressDelivery->getStreet()[0] . (empty($addressDelivery->getStreet()[1])
                ? ''
                : ', ' . $addressDelivery->getStreet()[1]),
            $addressDelivery->getCountryId(),
            $addressDelivery->getPostcode()
        );
        $cartData->setAddressBilling(
            $addressBilling->getCity(),
            $addressBilling->getFirstname() . ' ' . $addressBilling->getLastname(),
            $addressBilling->getTelephone(),
            $addressBilling->getStreet()[0] . (empty($addressBilling->getStreet()[1])
                ? ''
                : ', ' . $addressBilling->getStreet()[1]),
            $addressBilling->getCountryId(),
            $addressBilling->getPostcode(),
            $addressBilling->getVatId()
                ?: ($order->getData('customer_taxvat')
                ?: ($addressDelivery->getVatId()
                    ?: ''))
        );
        $cartData->setAmount(Util::convertAmountToFractional($order->getGrandTotal()));
        $cartData->setCreatedAt(strtotime($order->getCreatedAt()));

        $discount = abs($order->getBaseDiscountAmount());  //use abs because function getDiscountAmount return value with '-'.

        if ($discount > 0) {

            $cartData->setDiscount(
                0,
                'discount',
                Util::convertAmountToFractional($discount)
            );
        }

        return $cartData;
    }

    /**
     * @param array $customerEmail
     * @param Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param Magento\Sales\Model\OrderRepository $orderRepository
     * @param Magento\Sales\Model\ResourceModel\Order\Tax\Item $taxItem
     *
     * @return array
     */
    public function getPreviousOrders($customerEmail, $searchCriteriaBuilder, $orderRepository, $taxItem)
    {

        $orders = $searchCriteriaBuilder
            ->addFilter('customer_email', $customerEmail, 'eq')
            ->addFilter('status', [
                'complete',
                'closed',
            ], 'in')
            ->setPageSize(CartData::MAX_PREVIOUS_ORDERS)
            ->create();

        $orders = $orderRepository->getList($orders);

        if (empty($orders)) {
            return [];
        }

        $previousOrders = [];

        foreach ($orders as $order) {

            $previousOrders[] = Helper::getCart($order, $taxItem);
        }

        return $previousOrders;
    }

    /**
     * @param string $authorization_token
     * @param string $merchant_id
     * @param string $service_id
     * @param bool $is_sandbox
     * @param InfoInterface $payment
     * @param string $amount
     * @param mixed $that
     *
     * @return void|mixed
     * @throws LocalizedException
     */
    public static function refund($authorization_token, $merchant_id, $service_id, $is_sandbox, $payment, $amount, $that)
    {

        $api = new Api(
            $authorization_token,
            $merchant_id,
            $service_id,
            $is_sandbox
                ? Util::ENVIRONMENT_SANDBOX
                : Util::ENVIRONMENT_PRODUCTION);

        $order = $payment->getOrder();

        $refund = $api->createRefund(
            $api->prepareRefundData(
                Util::convertAmountToFractional($amount)
            ),
            $payment->getParentTransactionId()
        );

        if ($refund['success']) {

            if ($refund['body']['transaction']['status'] === 'settled') {
                return $that;
            }

            throw new LocalizedException(
                sprintf(
                    __('Refund for amount %s with UUID %s has been requested and waiting for completion.'),
                    $amount,
                    $order->getOrderCurrency()->getCode(),
                    $refund['body']['id']
                )
            );
        }

        throw new LocalizedException(__('The refund action is not available at this moment. Try again later.'));
    }
}