<?php

namespace Imoje\Paylater\Controller\Payment;

include_once __DIR__ . "/../../../Libs/autoload.php";

use Exception;
use Imoje\Payment\Util;
use Imoje\Payment\Notification as SdkNotification;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\CreditmemoFactory;
use Magento\Sales\Model\Order\Email\Sender\InvoiceSender;
use Magento\Sales\Model\Order\Invoice;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;
use Magento\Sales\Model\OrderRepository;
use Magento\Sales\Model\Service\CreditmemoService;
use Magento\Sales\Model\Service\InvoiceService;
use Magento\Store\Model\ScopeInterface;
use Magento\Sales\Api\OrderStatusHistoryRepositoryInterface;

/**
 * Class Notification
 *
 * @package Imoje\Paylater\Controller\Payment
 */
class Notification extends Action
{

	/**
	 * @var Order
	 */
	protected $order;

	/**
	 * @var ScopeConfigInterface
	 */
	protected $scopeConfig;

	/**
	 * @var BuilderInterface
	 */
	protected $builderInterface;

	/**
	 * @var OrderRepository
	 */
	protected $orderRepository;

	/**
	 * @var InvoiceService
	 */
	protected $invoiceService;

	/**
	 * @var \Magento\Framework\DB\Transaction
	 */
	protected $dbTransaction;

	/**
	 * @var InvoiceSender
	 */
	protected $invoiceSender;

	/**
	 * @var CreditmemoFactory
	 */
	protected $creditmemoFactory;

	/**
	 * @var CreditmemoService
	 */
	protected $creditmemoService;

	/**
	 * @var OrderStatusHistoryRepositoryInterface
	 */
	protected $orderStatusHistoryRepository;

	/**
	 * Constructor.
	 *
	 * @param Context                               $context
	 * @param Order                                 $order
	 * @param ScopeConfigInterface                  $scopeConfig
	 * @param BuilderInterface                      $builderInterface
	 * @param OrderRepositoryInterface              $orderRepository
	 * @param InvoiceService                        $invoiceService
	 * @param \Magento\Framework\DB\Transaction     $dbTransaction
	 * @param InvoiceSender                         $invoiceSender
	 * @param CreditmemoFactory                     $creditmemoFactory
	 * @param CreditmemoService                     $creditmemoService
	 * @param OrderStatusHistoryRepositoryInterface $orderStatusHistoryRepository
	 */
	public function __construct(
		Context                               $context,
		Order                                 $order,
		ScopeConfigInterface                  $scopeConfig,
		BuilderInterface                      $builderInterface,
		OrderRepositoryInterface              $orderRepository,
		InvoiceService                        $invoiceService,
		\Magento\Framework\DB\Transaction     $dbTransaction,
		InvoiceSender                         $invoiceSender,
		CreditmemoFactory                     $creditmemoFactory,
		CreditmemoService                     $creditmemoService,
		OrderStatusHistoryRepositoryInterface $orderStatusHistoryRepository
	) {
		parent::__construct($context);
		$this->order = $order;
		$this->scopeConfig = $scopeConfig;
		$this->builderInterface = $builderInterface;
		$this->orderRepository = $orderRepository;
		$this->invoiceService = $invoiceService;
		$this->dbTransaction = $dbTransaction;
		$this->invoiceSender = $invoiceSender;
		$this->creditmemoFactory = $creditmemoFactory;
		$this->creditmemoService = $creditmemoService;
		$this->orderStatusHistoryRepository = $orderStatusHistoryRepository;
	}

	/**
	 * @return void
	 * @throws Exception
	 */
	public function execute()
	{

		$notification = new SdkNotification(
			$this->getConfigValue('payment/imoje_paylater/service_id'),
			$this->getConfigValue('payment/imoje_paylater/service_key'),
			$this->getConfigValue('payment/imoje_paylater/debug_mode')
		);

		// it can be order data or notification code - depends on verification notification
		$resultCheckRequestNotification = $notification->checkRequest();

		if(is_int($resultCheckRequestNotification)) {

			echo $notification->formatResponse(SdkNotification::NS_ERROR, $resultCheckRequestNotification);
			exit();
		}

		$transactionStatuses = Util::getTransactionStatuses();

		if(!isset($transactionStatuses[$resultCheckRequestNotification['transaction']['status']])) {
			echo $notification->formatResponse(SdkNotification::NS_ERROR, SdkNotification::NC_UNHANDLED_STATUS);
			exit();
		}

		$order = $this->order->loadByIncrementId($resultCheckRequestNotification['transaction']['orderId']);

		if($order->getEntityId() === null) {

			echo $notification->formatResponse(SdkNotification::NS_ERROR, SdkNotification::NC_ORDER_NOT_FOUND);
			exit();
		}

		if($resultCheckRequestNotification['transaction']['type'] === SdkNotification::TRT_REFUND) {

			if($resultCheckRequestNotification['transaction']['status'] !== SdkNotification::TRS_SETTLED) {

				echo $notification->formatResponse(SdkNotification::NS_ERROR, SdkNotification::NC_IMOJE_REFUND_IS_NOT_SETTLED);
				exit();
			}

			$currentOrderStatus = $order->getState();

			if($currentOrderStatus === Order::STATE_CLOSED) {

				echo $notification->formatResponse(SdkNotification::NS_ERROR, SdkNotification::NC_ORDER_STATUS_IS_INVALID_FOR_REFUND);
				exit();
			}

			try {
				$creditmemo = $this->creditmemoFactory->createByOrder($order);
				$creditmemo->setGrandTotal(Util::convertAmountToMain($resultCheckRequestNotification['transaction']['amount']));
				$this->creditmemoService->refund($creditmemo);
				$this->creditmemoService->notify($creditmemo->getId());
			} catch(Exception $e) {

				echo $notification->formatResponse(SdkNotification::NS_ERROR, SdkNotification::NC_UNKNOWN);
				exit();
			}

			echo $notification->formatResponse(SdkNotification::NS_OK);
			exit();
		}

		if(!SdkNotification::checkRequestAmount($resultCheckRequestNotification, Util::convertAmountToFractional($order->getGrandTotal()), $order->getOrderCurrencyCode())) {

			echo $notification->formatResponse(SdkNotification::NS_ERROR, SdkNotification::NC_AMOUNT_NOT_MATCH);
			exit();
		}

		$currentOrderStatus = $order->getState();

		if($currentOrderStatus === Order::STATE_PROCESSING) {

			echo $notification->formatResponse(SdkNotification::NS_OK);
			exit();
		}

		switch($resultCheckRequestNotification['transaction']['status']) {
			case SdkNotification::TRS_SETTLED:
				$order->setTotalPaid(Util::convertAmountToMain($resultCheckRequestNotification['transaction']['amount']));
				$this->createTransaction($order, $resultCheckRequestNotification);
				$this->createInvoice($order, $resultCheckRequestNotification['transaction']['id']);
				$order->save();
				break;
			case SdkNotification::TRS_REJECTED:
				$orderState = Order::STATE_PAYMENT_REVIEW;
				$order->setState($orderState)->setStatus($orderState);
				$order->addStatusToHistory($orderState, __('The payment has been rejected.'));
				$order->save();
				break;
			default:
				break;
		}

		echo $notification->formatResponse(SdkNotification::NS_OK);
		exit();
	}

	/**
	 * @param Order $order
	 * @param array $payloadDecoded
	 *
	 * @return bool
	 */
	public function createTransaction($order, $payloadDecoded)
	{
		//get payment object from order object
		//get the object of builder class
		if(
			!($payment = $order->getPayment())
			|| !($trans = $this->builderInterface)
		) {
			return false;
		}

		$transaction = $trans->setPayment($payment)
			->setOrder($order)
			->setTransactionId($payloadDecoded['transaction']['id'])
			->setFailSafe(true)
			//build method creates the transaction and returns the object
			->build(Transaction::TYPE_CAPTURE);
		if(!$transaction) {
			return false;
		}

		$payment->addTransactionCommentsToOrder(
			$transaction,
			__('Paid amount: %1.', Util::convertAmountToMain($payloadDecoded['transaction']['amount']))
		);

		$payment->setTransactionId($payloadDecoded['transaction']['id']);

		$payment->setParentTransactionId(null);

		try {
			$payment->save();
			$transaction->save();

			return true;
		} catch(Exception $e) {
			return false;
		}
	}

	/**
	 * @param Order  $order
	 * @param string $transactionId
	 *
	 * @return bool
	 */
	public function createInvoice($order, $transactionId)
	{

		if(!$order->canInvoice()) {

			return false;
		}

		$invoice = $order->prepareInvoice();
		$invoice->setRequestedCaptureCase(Invoice::CAPTURE_ONLINE);
		$invoice->setTransactionId($transactionId);
		$invoice->register();
		$invoice->pay();
		$invoice->save();

		if(!$invoice->getEmailSent()) {

			$this->invoiceSender->send($invoice);

			$order->addRelatedObject($invoice);
			$order->addStatusHistoryComment(__('You notified customer about invoice #%1.', $invoice->getIncrementId()))
				->setIsCustomerNotified(true)
				->save();

			return true;
		}

		return false;
	}

	/**
	 * @param string $value
	 *
	 * @return string
	 */
	private function getConfigValue($value)
	{
		return $this->scopeConfig->getValue($value, ScopeInterface::SCOPE_STORE);
	}
}
