<?php

namespace Imoje\Paywall\Block;

include_once __DIR__ . "/../../Libs/autoload.php";

use Imoje\Payment\Helper;
use Imoje\Payment\Invoice;
use Imoje\Payment\Paywall;
use Imoje\Payment\Util;
use Imoje\Paywall\Model\Order;
use Magento\Checkout\Helper\Cart;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\App\ResponseFactory;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Sales\Model\OrderRepository;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Sales\Model\ResourceModel\Order\Tax\Item as TaxItem;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Tax\Api\TaxClassRepositoryInterface;

/**
 * Class Checkout
 *
 * @package Imoje\Paywall\Block
 */
class Checkout extends Template
{

	/**
	 * @var UrlInterface
	 */
	public $urlBuilder;

	/**
	 * @var ScopeConfigInterface
	 */
	protected $scopeConfig;

	/**
	 * @var OrderRepository
	 */
	protected $orderRepository;

	/**
	 * @var Registry
	 */
	private $registry;

	/**
	 * @var Order
	 */
	private $order;

	/**
	 * @var ResponseFactory
	 */
	private $responseFactory;

	/**
	 * @var OrderFactory
	 */
	private $searchCriteriaBuilder;

	/**
	 * @var ProductMetadataInterface $productMetadataInterface
	 */
	private $productMetadataInterface;

	/**
	 * @var TaxItem $taxItem
	 */
	private $taxItem;

	/**
	 * @var ProductRepositoryInterface\ $productRepository
	 */
	protected $productRepository;

	/**
	 * @var TaxClassRepositoryInterface $taxClassRepository
	 */
	protected $taxClassRepository;

	/**
	 * @var StoreInterface $store
	 */
	protected $store;

	/**
	 * Checkout constructor.
	 *
	 * @param Context                     $context
	 * @param Order                       $order
	 * @param Registry                    $registry
	 * @param StoreInterface              $store
	 * @param UrlInterface                $urlBuilder
	 * @param ScopeConfigInterface        $scopeConfig
	 * @param ResponseFactory             $responseFactory
	 * @param SearchCriteriaBuilder       $searchCriteriaBuilder
	 * @param OrderRepository             $orderRepository
	 * @param ProductMetadataInterface    $productMetadataInterface
	 * @param TaxItem                     $taxItem
	 * @param ProductRepositoryInterface  $productRepository
	 * @param TaxClassRepositoryInterface $taxClassRepository
	 */
	public function __construct(
		Context                     $context,
		Order                       $order,
		Registry                    $registry,
		StoreInterface              $store,
		UrlInterface                $urlBuilder,
		ScopeConfigInterface        $scopeConfig,
		ResponseFactory             $responseFactory,
		SearchCriteriaBuilder       $searchCriteriaBuilder,
		OrderRepository             $orderRepository,
		ProductMetadataInterface    $productMetadataInterface,
		TaxItem                     $taxItem,
		ProductRepositoryInterface  $productRepository,
		TaxClassRepositoryInterface $taxClassRepository
	) {
		parent::__construct($context);
		$this->registry = $registry;
		$this->order = $order;
		$this->store = $store;
		$this->urlBuilder = $urlBuilder;
		$this->scopeConfig = $scopeConfig;
		$this->responseFactory = $responseFactory;
		$this->searchCriteriaBuilder = $searchCriteriaBuilder;
		$this->orderRepository = $orderRepository;
		$this->productMetadataInterface = $productMetadataInterface;
		$this->taxItem = $taxItem;
		$this->productRepository = $productRepository;
		$this->taxClassRepository = $taxClassRepository;

		if($this->getOrderState($this->getOrderIdFromPost()) === \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT) {

			$this->redirectToHomePage();
			die();
		}
	}

	/**
	 * @param string $orderId
	 *
	 * @return mixed
	 */
	public function getOrderState($orderId = '')
	{
		return $this->getOrderObject($orderId)->getState();
	}

	/**
	 * Render form with order information prepared for Payment Page
	 *
	 * @param string $id
	 *
	 * @return bool|\Imoje\Paywall\Model\Sales\Order
	 */
	public function getOrderObject($id = '')
	{
		if(empty($id)) {
			$id = $this->getOrderId();
		}

		return $this->order->loadOrderById($id);
	}

	/**
	 * @return integer
	 */
	public function getOrderId()
	{
		return $this->registry->registry('orderId');
	}

	/**
	 * @return string
	 */
	public function getOrderIdFromPost()
	{
		$id = '';
		if(isset($_POST['orderId'])) {
			$id = $_POST['orderId'];
		}

		return $id;
	}

	/**
	 * That function redirect to homepage
	 *
	 * @return mixed
	 */
	public function redirectToHomePage()
	{
		return $this->responseFactory->create()->setRedirect($this->urlBuilder->getUrl())->sendResponse();
	}

	/**
	 * @param string $orderId
	 *
	 * @return string
	 */
	public function renderForm($orderId = '')
	{

		if(empty($orderId)) {
			$orderId = $this->getOrderId();
		}

		$order = $this->getOrderObject($orderId);

		$order->setState(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT);
		$order->save();

		$paywall = new Paywall(
			$this->getEnvironment(),
			$this->getConfigValue('payment/imoje_paywall/service_key'),
			$this->getConfigValue('payment/imoje_paywall/service_id'),
			$this->getConfigValue('payment/imoje_paywall/merchant_id')
		);

		$email = $order->getCustomerEmail();

		$billingAddress = $order->getBillingAddress();
		$addressFailure = $this->urlBuilder->getUrl('checkout/onepage/failure/');

		return $paywall->buildOrderForm(
			Paywall::prepareData(
				Util::convertAmountToFractional($order->getGrandTotal()),
				$order->getOrderCurrency()->getCode(),
				$order->getIncrementId(),
				$billingAddress->getFirstname(),
				$billingAddress->getLastname(),
				'',
				$email,
				$billingAddress->getTelephone(),
				$this->urlBuilder->getUrl('checkout/onepage/success/'),
				$addressFailure,
				$addressFailure,
				$this->getConfigValue('payment/imoje_paywall/version') . ';magento2_' . $this->productMetadataInterface->getVersion(),
				'',
				'',
				$this->getConfigValue('payment/imoje_paywall/ing_ksiegowosc')
					? Invoice::get(
					Helper::getCart(
						$order,
						$this->taxItem,
						$this->productRepository,
						$this->taxClassRepository
					),
					$email,
					false
				)
					: ''

			),
			__('Continue')
		);
	}

	/**
	 * @return string
	 */
	public function getEnvironment()
	{
		return $this->getConfigValue('payment/imoje_paywall/sandbox') == 1
			? Util::ENVIRONMENT_SANDBOX
			: Util::ENVIRONMENT_PRODUCTION;
	}

	/**
	 * @param string $value
	 *
	 * @return string
	 */
	private function getConfigValue($value)
	{
		return $this->scopeConfig->getValue($value, ScopeInterface::SCOPE_STORE);
	}
}
