<?php

namespace Imoje\Pbl\Block;

include_once __DIR__ . "/../../Libs/autoload.php";

use Imoje\Payment\Api;
use Imoje\Payment\Helper;
use Imoje\Payment\Invoice;
use Imoje\Payment\Util;
use Imoje\Pbl\Model\Order;
use Magento\Checkout\Helper\Cart;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\App\ResponseFactory;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\ScopeInterface;
use Zend\Log\Logger;
use Zend\Log\Writer\Stream;
use Magento\Sales\Model\ResourceModel\Order\Tax\Item as TaxItem;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Tax\Api\TaxClassRepositoryInterface;

/**
 * Class Checkout
 *
 * @package Imoje\Pbl\Block
 */
class Checkout extends Template
{

	/**
	 * @var UrlInterface
	 */
	public $urlBuilder;

	/**
	 * @var ScopeConfigInterface
	 */
	protected $scopeConfig;

	/**
	 * @var Registry
	 */
	private $registry;

	/**
	 * @var Order
	 */
	private $order;

	/**
	 * @var StoreInterface
	 */
	private $store;

	/**
	 * @var Cart
	 */
	private $cartHelper;

	/**
	 * @var ResponseFactory
	 */
	private $responseFactory;

	/**
	 * @var ProductMetadataInterface $productMetadataInterface
	 */
	private $productMetadataInterface;

	/**
	 * @var TaxItem $taxItem
	 */
	private $taxItem;

	/**
	 * @var ProductRepositoryInterface\ $productRepository
	 */
	protected $productRepository;

	/**
	 * @var TaxClassRepositoryInterface $taxClassRepository
	 */
	protected $taxClassRepository;

	/**
	 * Checkout constructor.
	 *
	 * @param Context                     $context
	 * @param Order                       $order
	 * @param Registry                    $registry
	 * @param StoreInterface              $store
	 * @param UrlInterface                $urlBuilder
	 * @param ScopeConfigInterface        $scopeConfig
	 * @param Cart                        $cartHelper
	 * @param ResponseFactory             $responseFactory
	 * @param ProductMetadataInterface    $productMetadataInterface
	 * @param TaxItem                     $taxItem
	 * @param ProductRepositoryInterface  $productRepository
	 * @param TaxClassRepositoryInterface $taxClassRepository
	 */
	public function __construct(
		Context                     $context,
		Order                       $order,
		Registry                    $registry,
		StoreInterface              $store,
		UrlInterface                $urlBuilder,
		ScopeConfigInterface        $scopeConfig,
		Cart                        $cartHelper,
		ResponseFactory             $responseFactory,
		ProductMetadataInterface    $productMetadataInterface,
		TaxItem                     $taxItem,
		ProductRepositoryInterface  $productRepository,
		TaxClassRepositoryInterface $taxClassRepository
	) {
		parent::__construct($context);
		$this->registry = $registry;
		$this->order = $order;
		$this->store = $store;
		$this->urlBuilder = $urlBuilder;
		$this->scopeConfig = $scopeConfig;
		$this->cartHelper = $cartHelper;
		$this->responseFactory = $responseFactory;
		$this->productMetadataInterface = $productMetadataInterface;
		$this->taxItem = $taxItem;
		$this->productRepository = $productRepository;
		$this->taxClassRepository = $taxClassRepository;

		if($this->getOrderState($this->getOrderIdFromPost()) === \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT) {

			$this->redirectToHomePage();
			die();
		}
	}

	/**
	 * @param string $orderId
	 *
	 * @return mixed
	 */
	public function getOrderState($orderId = '')
	{
		return $this->getOrderObject($orderId)->getState();
	}

	/**
	 * Render form with order information prepared for Payment Page
	 *
	 * @param string $id
	 *
	 * @return bool|\Imoje\Pbl\Model\Sales\Order
	 */
	public function getOrderObject($id = '')
	{
		if(empty($id)) {
			$id = $this->getOrderId();
		}

		return $this->order->loadOrderById($id);
	}

	/**
	 * @return integer
	 */
	public function getOrderId()
	{
		return $this->registry->registry('orderId');
	}

	/**
	 * @return string
	 */
	public function getOrderIdFromPost()
	{
		$id = '';
		if(isset($_POST['orderId'])) {
			$id = $_POST['orderId'];
		}

		return $id;
	}

	/**
	 * That function redirect to homepage
	 *
	 * @return mixed
	 */
	public function redirectToHomePage()
	{
		return $this->responseFactory->create()->setRedirect($this->urlBuilder->getUrl())->sendResponse(); //Redirect to cms page
	}

	/**
	 * @return array|string|void
	 */
	public function renderPbl()
	{

		$isPblPassed = $this->isPblPassed();

		if($isPblPassed['status']) {

			return $this->renderForm($this->getOrderId(), $isPblPassed['body']['pm'], $isPblPassed['body']['pmc']);
		}

		$writer = new Stream(BP . '/var/log/imoje.log');
		$this->logger = new Logger();
		$this->logger->addWriter($writer);

		$order = $this->getOrderObject($this->getOrderId());

		$imojeApi = new Api(
			$this->getConfigValue('payment/imoje_pbl/authorization_token'),
			$this->getConfigValue('payment/imoje_pbl/merchant_id'),
			$this->getConfigValue('payment/imoje_pbl/service_id'),
			$this->getConfigValue('payment/imoje_pbl/sandbox')
				? Util::ENVIRONMENT_SANDBOX
				: Util::ENVIRONMENT_PRODUCTION
		);

		$service = $imojeApi->getServiceInfo();

		if(!$service['success']) {

			$this->logger->info('Bad response in api, errors: '
				. $service['data']['body']);
			$this->redirectToHomePage();
			die();
		}

		if(!isset($service['body']['service']['isActive']) && $service['body']['service']['isActive']) {

			$this->logger->info('Service is inactive in imoje');
			$this->redirectToHomePage();
			die();
		}

		$currency = $order->getOrderCurrency()->getCode();
		$service = $service['body']['service'];

		$paymentMethodList = [];

		foreach($service['paymentMethods'] as $paymentMethod) {

			$pm = strtolower($paymentMethod['paymentMethod']);

			if($paymentMethod['isActive']
				&& (
					($pm === Util::getPaymentMethod('pbl'))
					|| ($pm === Util::getPaymentMethod('ing'))
				)
				&& strtolower($paymentMethod['currency']) === strtolower($currency)

				// TODO: uncomment when api will be fine with limits

				//                && (
				//                    isset($paymentMethod['transactionLimits']['maxTransaction']['value'])
				//                    && $paymentMethod['transactionLimits']['maxTransaction']['value']
				//                )
				//                && $paymentMethod['transactionLimits']['maxTransaction']['value'] > $cartTotal
				//                && (
				//                    isset($paymentMethod['transactionLimits']['minTransaction']['value'])
				//                    && $paymentMethod['transactionLimits']['minTransaction']['value']
				//                )
				//                && $paymentMethod['transactionLimits']['minTransaction']['value'] < $cartTotal
			) {

				$logo = Util::getPaymentMethodCodeLogo($paymentMethod['paymentMethodCode']);

				foreach($paymentMethod['paymentMethodCodeImage'] as $imgList) {
					if(isset($imgList['png']) && $imgList['png']) {
						$logo = $imgList['png'];
					}
				}

				$paymentMethodList[] = [
					'paymentMethod'     => $paymentMethod['paymentMethod'],
					'paymentMethodCode' => $paymentMethod['paymentMethodCode'],
					'description'       => $paymentMethod['description'],
					'isAvailable'       => $paymentMethod['isOnline'],
					'logo'              => $logo,
				];
			}
		}

		return $paymentMethodList;
	}

	/**
	 * @return array
	 */
	public function isPblPassed()
	{
		$pm = $this->getRequest()->getParam('pm');
		$pmc = $this->getRequest()->getParam('pmc');

		if($pm && $pmc) {

			return [
				'status' => true,
				'body'   => [
					'pm'  => $this->getRequest()->getParam('pm'),
					'pmc' => $this->getRequest()->getParam('pmc'),
				],
			];
		}

		return [
			'status' => false,
			'body'   => [],
		];
	}

	/**
	 * @param string $orderId
	 *
	 * @return string
	 */
	public function renderForm($orderId, $pm = '', $pmc = '')
	{

		$imojeApi = new Api(
			$this->getConfigValue('payment/imoje_pbl/authorization_token'),
			$this->getConfigValue('payment/imoje_pbl/merchant_id'),
			$this->getConfigValue('payment/imoje_pbl/service_id'),
			$this->getConfigValue('payment/imoje_pbl/sandbox')
				? Util::ENVIRONMENT_SANDBOX
				: Util::ENVIRONMENT_PRODUCTION
		);

		$service = $imojeApi->getServiceInfo();

		if(!$service['success']) {

			$this->logger->info('Bad response in api, errors: '
				. $service['data']['body']);
			$this->redirectToHomePage();
			die();
		}

		if(!isset($service['body']['service']['isActive']) && $service['body']['service']['isActive']) {

			$this->logger->info('Service is inactive in imoje');
			$this->redirectToHomePage();
			die();
		}

		$paymentMethodList = $service['body']['service']['paymentMethods'];

		$isValid = false;

		foreach($paymentMethodList as $paymentMethod) {

			if(($paymentMethod['paymentMethodCode'] === $pmc)
				&& ($paymentMethod['paymentMethod'] === $pm)
				&& $paymentMethod['isActive']
				&& $paymentMethod['isOnline']) {

				$isValid = true;
			}
		}

		$writer = new Stream(BP . '/var/log/imoje.log');
		$this->logger = new Logger();
		$this->logger->addWriter($writer);

		if(!$isValid) {
			$this->logger->info(
				'Payment channel is inactive or offline or was not found. Code: ' . $pmc . ', Method: ' . $pm . '.'
			);
			$this->redirectToHomePage();
			die();
		}

		$order = $this->getOrderObject($orderId);

		$email = $order->getCustomerEmail();

		$billingAddress = $order->getBillingAddress();

		$prepareData = $imojeApi->prepareData(
			$order->getGrandTotal(),
			$order->getOrderCurrency()->getCode(),
			$order->getIncrementId(),
			$pm,
			$pmc,
			$this->urlBuilder->getUrl('checkout/onepage/success/'),
			$this->urlBuilder->getUrl('checkout/onepage/failure/'),
			$billingAddress->getFirstname(),
			$billingAddress->getLastname(),
			$email,
			Api::TRANSACTION_TYPE_SALE,
			'',
			'',
			[],
			'',
			$this->getConfigValue('payment/imoje_pbl/ing_ksiegowosc')
				? Invoice::get(
				Helper::getCart(
					$order,
					$this->taxItem,
					$this->productRepository,
					$this->taxClassRepository
				),
				$email,
				true
			)
				: ''
		);

		$transaction = $imojeApi->createTransaction($prepareData);

		$order->setState(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT);
		$order->save();

		if($transaction['success']) {

			return $imojeApi->buildOrderForm($transaction,
				__('Continue')
			);
		}

		if(isset($transaction['data']['body']) && $transaction['data']['body']) {
			$this->logger->info('Transaction could not be initialized, error: ' . $transaction['data']['body']);
		}

		$this->redirectToHomePage();
		die();
	}

	/**
	 * @param string $value
	 *
	 * @return string
	 */
	private function getConfigValue($value)
	{
		return $this->scopeConfig->getValue($value, ScopeInterface::SCOPE_STORE);
	}
}
