<?php

namespace Imoje\Pragmago\Block;

include_once __DIR__ . "/../../Libs/autoload.php";

use Imoje\Pragmago\Model\Order;
use Imoje\Payment\Api;
use Imoje\Payment\Helper;
use Imoje\Payment\Invoice;
use Imoje\Payment\Util;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResponseFactory;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Sales\Model\OrderRepository;
use Magento\Store\Model\ScopeInterface;
use Zend\Log\Logger;
use Zend\Log\Writer\Stream;
use Magento\Sales\Model\ResourceModel\Order\Tax\Item as TaxItem;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Tax\Api\TaxClassRepositoryInterface;

/**
 * Class Checkout
 *
 * @package Imoje\Pragmago\Block
 */
class Checkout extends Template
{

	/**
	 * @var UrlInterface
	 */
	public $urlBuilder;

	/**
	 * @var ScopeConfigInterface
	 */
	protected $scopeConfig;

	/**
	 * @var OrderRepository
	 */
	protected $orderRepository;

	/**
	 * @var Registry
	 */
	private $registry;

	/**
	 * @var Order
	 */
	private $order;

	/**
	 * @var ResponseFactory
	 */
	private $responseFactory;

	/**
	 * @var TaxItem $taxItem
	 */
	private $taxItem;

	/**
	 * @var ProductRepositoryInterface\ $productRepository
	 */
	protected $productRepository;

	/**
	 * @var TaxClassRepositoryInterface $taxClassRepository
	 */
	protected $taxClassRepository;

	/**
	 * Checkout constructor.
	 *
	 * @param Context                     $context
	 * @param Order                       $order
	 * @param Registry                    $registry
	 * @param UrlInterface                $urlBuilder
	 * @param ScopeConfigInterface        $scopeConfig
	 * @param ResponseFactory             $responseFactory
	 * @param OrderRepository             $orderRepository
	 * @param TaxItem                     $taxItem
	 * @param ProductRepositoryInterface  $productRepository
	 * @param TaxClassRepositoryInterface $taxClassRepository
	 */
	public function __construct(
		Context                     $context,
		Order                       $order,
		Registry                    $registry,
		UrlInterface                $urlBuilder,
		ScopeConfigInterface        $scopeConfig,
		ResponseFactory             $responseFactory,
		OrderRepository             $orderRepository,
		TaxItem                     $taxItem,
		ProductRepositoryInterface  $productRepository,
		TaxClassRepositoryInterface $taxClassRepository
	) {
		parent::__construct($context);
		$this->registry = $registry;
		$this->order = $order;
		$this->urlBuilder = $urlBuilder;
		$this->scopeConfig = $scopeConfig;
		$this->responseFactory = $responseFactory;
		$this->orderRepository = $orderRepository;
		$this->taxItem = $taxItem;
		$this->productRepository = $productRepository;
		$this->taxClassRepository = $taxClassRepository;

		if($this->getOrderState($this->getOrderIdFromPost()) === \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT) {

			$this->redirectToHomePage();
			die();
		}
	}

	/**
	 * @param string $orderId
	 *
	 * @return mixed
	 */
	public function getOrderState($orderId = '')
	{
		return $this->getOrderObject($orderId)->getState();
	}

	/**
	 * Render form with order information prepared for Payment Page
	 *
	 * @param string $id
	 *
	 * @return bool|\Imoje\Pragmago\Model\Sales\Order
	 */
	public function getOrderObject($id = '')
	{
		if(empty($id)) {
			$id = $this->getOrderId();
		}

		return $this->order->loadOrderById($id);
	}

	/**
	 * @return integer
	 */
	public function getOrderId()
	{
		return $this->registry->registry('orderId');
	}

	/**
	 * @return string
	 */
	public function getOrderIdFromPost()
	{
		$id = '';
		if(isset($_POST['orderId'])) {
			$id = $_POST['orderId'];
		}

		return $id;
	}

	/**
	 * That function redirect to homepage
	 *
	 * @return mixed
	 */
	public function redirectToHomePage()
	{
		return $this->responseFactory->create()->setRedirect($this->urlBuilder->getUrl())->sendResponse(); //Redirect to cms page
	}

	/**
	 * @return string
	 */
	public function renderForm()
	{

		$writer = new Stream(BP . '/var/log/imoje.log');
		$this->logger = new Logger();
		$this->logger->addWriter($writer);

		$imojeApi = new Api(
			$this->getConfigValue('payment/imoje_pragmago/authorization_token'),
			$this->getConfigValue('payment/imoje_pragmago/merchant_id'),
			$this->getConfigValue('payment/imoje_pragmago/service_id'),
			$this->getEnvironment()
		);

		$order = $this->getOrderObject($this->getOrderId());

		$email = $order->getCustomerEmail();

		$billingAddress = $order->getBillingAddress();

		$prepareData = $imojeApi->prepareData(
			$order->getGrandTotal(),
			$order->getOrderCurrency()->getCode(),
			$order->getIncrementId(),
			Util::getPaymentMethod('paylater'),
			Util::getPaymentMethodCode('pragma_go'),
			$this->urlBuilder->getUrl('checkout/onepage/success/'),
			$this->urlBuilder->getUrl('checkout/onepage/failure/'),
			$billingAddress->getFirstname(),
			$billingAddress->getLastname(),
			$email,
			Api::TRANSACTION_TYPE_SALE,
			'',
			'',
			$this->getAddress($order),
			'',
			$this->getConfigValue('payment/imoje_pragmago/ing_ksiegowosc')
				? Invoice::get(
				Helper::getCart(
					$order,
					$this->taxItem,
					$this->productRepository,
					$this->taxClassRepository
				),
				$email,
				true
			)
				: ''
		);

		$transaction = $imojeApi->createTransaction($prepareData);

		$order->setState(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT);
		$order->save();

		if($transaction['success']) {

			return $imojeApi->buildOrderForm($transaction,
				__('Continue')
			);
		}

		if(isset($transaction['data']['body']) && $transaction['data']['body']) {
			$this->logger->info('Transaction could not be initialized, error: ' . $transaction['data']['body']);
		}

		$this->redirectToHomePage();
		die();
	}

	/**
	 * @param string $value
	 *
	 * @return string
	 */
	private function getConfigValue($value)
	{
		return $this->scopeConfig->getValue($value, ScopeInterface::SCOPE_STORE);
	}

	/**
	 * @return string
	 */
	public function getEnvironment()
	{
		return $this->getConfigValue('payment/imoje_pragmago/sandbox') == 1
			? Util::ENVIRONMENT_SANDBOX
			: Util::ENVIRONMENT_PRODUCTION;
	}

	/**
	 * @param object $order
	 *
	 * @return array
	 */
	private function getAddress($order)
	{

		$addressBilling = $order->getBillingAddress();
		$addressBillingStreet = $addressBilling->getStreet();

		$address = [
			'billing' => \Imoje\Payment\Api::prepareAddressData(
				$addressBilling->getFirstname(),
				$addressBilling->getLastname(),
				$addressBillingStreet[0] . (empty($addressBillingStreet[1])
					? ''
					: ', ' . $addressBillingStreet[1]),
				$addressBilling->getCity(),
				'',
				$addressBilling->getPostcode(),
				$addressBilling->getCountryId()
			),
		];

		if($order->hasShipments()) {
			$addressDelivery = $order->getShippingAddress();
			$addressDeliveryStreet = $addressDelivery->getStreet();

			$address['shipping'] = \Imoje\Payment\Api::prepareAddressData(
				$addressDelivery->getFirstname(),
				$addressDelivery->getLastname(),
				$addressDeliveryStreet[0] . (empty($addressDeliveryStreet[1])
					? ''
					: ', ' . $addressDeliveryStreet[1]),
				$addressDelivery->getCity(),
				'',
				$addressDelivery->getPostcode(),
				$addressDelivery->getCountryId()
			);
		}

		return $address;
	}
}
